package com.lqkj.cmlcp.message;
import lombok.Data;

/**
 * 全局统一返回结果类
 */
@Data
//@ApiModel(value = "全局统一返回结果")
public class Result<T> {

    //@ApiModelProperty(value = "返回码")
    private Integer code;

    //@ApiModelProperty(value = "返回消息")
    private String message;

    //@ApiModelProperty(value = "返回数据")
    private T data;

    private Boolean success = true;    // 表示当前操作是成功还是失败 true:成功 false:失败

    public Result(){}

    protected static <T> Result<T> build(T data) {
            Result<T> result = new Result<T>();
            if (data != null)
            result.setData(data);
        return result;
    }

    public static <T> Result<T> build(T body, ResultCodeEnum resultCodeEnum, Boolean isSuccess) {
        Result<T> result = build(body);
        result.setCode(resultCodeEnum.getCode());
        result.setMessage(resultCodeEnum.getMessage());
        result.setSuccess(isSuccess);
        return result;
    }

    public static <T> Result<T> build(Integer code, String message, Boolean isSuccess) {
        Result<T> result = build(null);
        result.setCode(code);
        result.setMessage(message);
        result.setSuccess(isSuccess);
        return result;
    }

    private static <T> Result<T> build(T data, String msg, boolean isSuccess) {
        Result<T> result = build(data);
        result.setCode(200);
        result.setMessage(msg);
        result.setSuccess(isSuccess);
        return result;
    }

    public static<T> Result<T> ok(){
        return Result.ok(null);
    }

    public static <T> Result<T> me(Integer code, String message, Boolean isSuccess){
        return build(code, message, isSuccess);
    }

    /**
     * 操作成功
     * @param data
     * @param <T>
     * @return
     */
    public static<T> Result<T> ok(T data){
        return build(data, ResultCodeEnum.SUCCESS, true);
    }

    public static<T> Result<T> ok(T data, String msg){
        return build(data, msg, true);
    }



    public static<T> Result<T> fail(){
        return Result.fail(null);
    }

    /**
     * 操作失败
     * @param data
     * @param <T>
     * @return
     */
    public static<T> Result<T> fail(T data){
        Result<T> result = build(data);
        return build(data, ResultCodeEnum.FAIL, false);
    }

    public Result<T> message(String msg){
        this.setMessage(msg);
        return this;
    }

    public Result<T> code(Integer code){
        this.setCode(code);
        return this;
    }

    public boolean isOk() {
        if(this.getCode().intValue() == ResultCodeEnum.SUCCESS.getCode().intValue()) {
            return true;
        }
        return false;
    }
}
