package com.lqkj.cmlcp.module.authority.service.impl;

import com.lqkj.cmlcp.module.authority.domain.UserInfo;
import com.lqkj.cmlcp.module.authority.mapper.UserInfoMapper;
import com.lqkj.cmlcp.module.authority.service.UserInfoService;
import com.mybatisflex.core.query.QueryWrapper;
import com.mybatisflex.core.update.UpdateChain;
import com.mybatisflex.spring.service.impl.ServiceImpl;
import jakarta.annotation.PostConstruct;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;
import java.util.Objects;

@Component
public class UserInfoServiceImpl extends ServiceImpl<UserInfoMapper, UserInfo> implements UserInfoService{

    @Autowired
    private UserInfoMapper userInfoMapper;


    @Override
    public UserInfo findByUserCode(String userCode) {
        return userInfoMapper.selectOneByQuery(new QueryWrapper().eq("user_code",userCode));
    }

        /**
     * 账号是否锁定
     * @param userCode
     * @return
     */
    @Transactional
    public boolean isLocked(String userCode) {
        UserInfo userInfo = findByUserCode(userCode);
        if (userInfo == null){
            return false;
        }else {
            if (userInfo.getLocking()) {
                if (new Date().getTime() - userInfo.getLockTime().getTime() > 1000 * 60 * 60){
                    UpdateChain.of(UserInfo.class)
                            .set(UserInfo::getLocking, false)
                            .set(UserInfo::getLockTime, null)
                            .set(UserInfo::getLoginErrorCount, 0)
                            .where(UserInfo::getId).eq(userInfo.getId())
                            .update();
                    return false;
                }else{
                    return true;
                }
            }else{
                return false;
            }
        }
    }

    /**
     * 更新登录错误次数
     * @param userCode
     * @return
     */
    @Transactional
    public String lockedUser(String userCode){
        UserInfo userInfo = findByUserCode(userCode);
        if (userInfo == null){
            return "用户名或密码错误";
        }else{
            int loginErrorCount = userInfo.getLoginErrorCount() == null ? 0 : userInfo.getLoginErrorCount();
            loginErrorCount += 1;
            userInfo.setLoginErrorCount(loginErrorCount);
            if (loginErrorCount == 4){
                userInfo.setLocking(true);
                userInfo.setLockTime(new Date());
                userInfoMapper.update(userInfo);
                return "账号已被冻结,请稍后重试";
            }
            userInfoMapper.update(userInfo);
            return "账号、密码或授权码输入错误," + (4 - loginErrorCount) +"次错误后,账号将会冻结1小时";
        }
    }

    /**
     * 解锁用户
     * @param userCode
     */
    @Transactional
    public void unlockedUser(String userCode){
        UserInfo userInfo = findByUserCode(userCode);
        UpdateChain.of(UserInfo.class)
                .set(UserInfo::getLocking, false)
                .set(UserInfo::getLockTime, null)
                .set(UserInfo::getLoginErrorCount, 0)
                .where(UserInfo::getId).eq(userInfo.getId())
                .update();
    }

    @Override
    public UserInfo selectByUserName(String username) {
        return userInfoMapper.selectOneByQuery(new QueryWrapper().eq("user_name",username));
    }


    @PostConstruct
    @Transactional
    public void initSuperUser() {
        if (Objects.isNull(userInfoMapper.selectOneById(1))){
            initSuperAdmin();
            initUserIdSeq();
            initSuperAdminRole();
        }
    }

    private void initSuperAdmin() {
        UserInfo userInfo = new UserInfo()
                .setId(1)
                .setUserCode("admin")
                .setDisplayName("超级管理员")
                .setPassword("$2a$10$OIBK8kKBDY07OS4CcVdx9e2GdxKH4T2lqWfDXgubjwZbxbis5.vSu")
                .setAuthorizationCode("87449007")
                .setHeadImg("")
                .setFileSavePath("")
                .setHasManage(true)
                .setHasAuth(true)
                .setLocking(false)
                .setLockTime(null)
                .setLoginErrorCount(0)
                .setUpdateTime(new Date())
                .setEnabled(true);
        userInfoMapper.insert(userInfo);

    }

    private void initUserIdSeq() {
        userInfoMapper.initUserIdSeq();
    }

    private void initSuperAdminRole() {

        userInfoMapper.initSuperAdminRole();
    }

    private void test(){



    }

}