package com.lqkj.cmlcp.module.jwt.controller;

import com.lqkj.cmlcp.message.MessageBean;
import com.lqkj.cmlcp.message.Result;
import com.lqkj.cmlcp.module.authority.domain.LoginBody;
import com.lqkj.cmlcp.module.authority.domain.UserInfo;
import com.lqkj.cmlcp.module.authority.service.UserInfoService;
import com.lqkj.cmlcp.module.jwt.service.JwtService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.media.Content;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.servlet.http.HttpServletRequest;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

@RestController
@RequestMapping("/jwt")
@Tag(name = "授权服务", description = "授权服务")
public class JwtController {

    @Autowired
    private  UserInfoService userInfoService;
    private final JwtService jwtService;
    private final PasswordEncoder passwordEncoder;

    public JwtController(JwtService jwtService, PasswordEncoder passwordEncoder) {
        this.jwtService = jwtService;
        this.passwordEncoder = passwordEncoder;
    }

    @Operation(
            summary = "5.1.1.2 登录接口",
            description = "5.1.1.2 登录接口",
            requestBody = @io.swagger.v3.oas.annotations.parameters.RequestBody(
                    description = "登录信息对象",
                    required = true,
                    content = @Content(
                            mediaType = "application/json",
                            schema = @Schema(implementation = LoginBody.class)
                    )
            ))
    @PostMapping("/token")
    public Result<String> getToken(@RequestBody LoginBody loginBody){
        String userCode = loginBody.getUsername();
        String password = loginBody.getPassword();
        String authCode = loginBody.getAuthCode();
//        if (userCode.length() != 172 || password.length() != 172 || authCode.length() != 172) {
//            return Result.fail("用户名或密码或授权码错误！");
//        }
//        userCode = RSAUtils.decryptBase64(userCode);
//        password = RSAUtils.decryptBase64(password);
//        authCode = RSAUtils.decryptBase64(authCode);

        UserInfo userInfo = userInfoService.findByUserCode(userCode);
        if (userInfo == null) {
            return Result.fail("账号不存在！");
        }
        if (userInfoService.isLocked(userInfo.getUserCode())) {//账号是否被冻结
            return Result.fail("账号已被冻结,请稍后重试！");
        }
        if (!passwordEncoder.matches(password, userInfo.getPassword())) {
            return Result.fail(userInfoService.lockedUser(userInfo.getUserCode()));
        }
        if (!authCode.equals(userInfo.getAuthorizationCode())) {
            return Result.fail("授权码错误！");
        }
        userInfoService.unlockedUser(userInfo.getUserCode());

        return Result.ok(jwtService.generateToken(userCode));
    }

    @Operation(
            summary = "5.1.1.4 刷新token接口",
            description = "5.1.1.4 刷新token接口"
    )
    @PostMapping("/refreshToken")
    public MessageBean<String> refreshToken(HttpServletRequest request) {
        String authHeader = request.getHeader("Authorization");
        String username = jwtService.decryptUsernameWithHeader(authHeader);
        return MessageBean.ok(jwtService.generateToken(username), "刷新token");

    }

    @Operation(
            summary = "直接获取ADMIN-TOKEN",
            description = "直接获取ADMIN-TOKEN"
    )
    @PostMapping("/getAdminToken")
    public Result<String> getAdminToken() {
        return Result.ok(jwtService.generateToken("meta-link"), "获取ADMIN-TOKEN");
    }
}
