package com.lqkj.cmlcp.utils;

import javax.crypto.Cipher;
import java.nio.charset.StandardCharsets;
import java.security.*;
import java.security.spec.InvalidKeySpecException;
import java.security.spec.X509EncodedKeySpec;
import java.util.Base64;

/**
 * @ClassName RSAUtils
 * @Description TODO
 * @Author wells
 * @Date 2021/3/2 11:37 上午
 * @Version 1.0
 **/
public class RSAUtils {
    //KeyPair is a simple holder for a key pair.
    private static final KeyPair keyPair = initKey();



    /**
     * 初始化方法，产生key pair，提供provider和random
     *
     * @return KeyPair instance
     */
    private static KeyPair initKey() {
        try {
            //添加provider
            //产生用于安全加密的随机数
            SecureRandom random = new SecureRandom();
            KeyPairGenerator generator = KeyPairGenerator.getInstance("RSA");
            generator.initialize(1024, random);
            return generator.generateKeyPair();
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 产生public key
     *
     * @return public key字符串
     */
    public static String generateBase64PublicKey() {
        PublicKey publicKey = keyPair.getPublic();
        //encodeBase64(): Encodes binary data using the base64
        //algorithm but does not chunk the output.
        //getEncoded():返回key的原始编码形式
        return Base64.getEncoder().encodeToString(publicKey.getEncoded());
    }

    /**
     * 解密数据
     *
     * @param string 需要解密的字符串
     * @return 破解之后的字符串
     */
    public static String decryptBase64(String string) {
        //decodeBase64():将Base64数据解码为"八位字节”数据
        return new String(decrypt(Base64.getDecoder().decode(string.getBytes())));
    }
    public static String encryptBase64(String str) {
        return Base64.getEncoder().encodeToString(encrypt(str.getBytes(StandardCharsets.UTF_8)));
    }

    public static String encryptBase64(String str, String publicKey) {
        return Base64.getEncoder().encodeToString(encrypt(str.getBytes(StandardCharsets.UTF_8), publicKey));
    }

    private static byte[] encrypt(byte[] byteArray){
        try {

            Cipher cipher = Cipher.getInstance("RSA");
            PublicKey publicKey = keyPair.getPublic();
            cipher.init(Cipher.ENCRYPT_MODE, publicKey);
            return cipher.doFinal(byteArray);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    private static byte[] decrypt(byte[] byteArray) {
        try {

            //Cipher: 提供加密和解密功能的实例
            //transformation: "algorithm/mode/padding"
            Cipher cipher = Cipher.getInstance("RSA");
            PrivateKey privateKey = keyPair.getPrivate();
            //初始化
            cipher.init(Cipher.DECRYPT_MODE, privateKey);
            //doFinal(): 加密或者解密数据
            return cipher.doFinal(byteArray);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    private static byte[] encrypt(byte[] byteArray, String publicKey){
        try {

            Cipher cipher = Cipher.getInstance("RSA");
            cipher.init(Cipher.ENCRYPT_MODE, getPublicKey(publicKey));
            return cipher.doFinal(byteArray);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    private static PublicKey getPublicKey(String key) throws NoSuchAlgorithmException, InvalidKeySpecException {
        byte[] keyBytes = Base64.getDecoder().decode(key);
        X509EncodedKeySpec keySpec = new X509EncodedKeySpec(keyBytes);
        KeyFactory keyFactory = KeyFactory.getInstance("RSA");
        return keyFactory.generatePublic(keySpec);
    }

    public static void main(String[] args) {
        String s = RSAUtils.encryptBase64("9wytyRIgskPdprmHvb13kJJpUv78=");
        System.out.println(s);
        String a = RSAUtils.decryptBase64(s);
        System.out.println(a);
    }
}
