create table authority_info (
                                authority_id         INT4                 not null,
                                parent_id            INT4                 null,
                                authority_name       VARCHAR(255)         null,
                                en_name              VARCHAR(255)         null,
                                route                VARCHAR(255)         null,
                                component            VARCHAR(255)         null,
                                redirect             VARCHAR(255)         null,
                                icon                 VARCHAR(255)         null,
                                order_id             INT4                 null,
                                constraint PK_AUTHORITY_INFO primary key (authority_id)
);

comment on table authority_info is
'权限信息：authority_info';

comment on column authority_info.authority_id is
'权限ID：authority_id';

comment on column authority_info.parent_id is
'父权限ID：parent_id';

comment on column authority_info.authority_name is
'权限名称：authority_name';

comment on column authority_info.en_name is
'英文名：en_name';

comment on column authority_info.route is
'路由地址：route';

comment on column authority_info.component is
'组件：component';

comment on column authority_info.redirect is
'重定向地址：redirect';

comment on column authority_info.icon is
'图标：icon';

comment on column authority_info.order_id is
'排序：order_id';

create table role_authority (
                                role_id              INT4                 not null,
                                authority_id         INT4                 not null,
                                constraint PK_ROLE_AUTHORITY primary key (role_id, authority_id)
);
comment on table role_authority is
'角色权限信息：role_authority';

comment on column role_authority.role_id is
'角色ID：role_id';

comment on column role_authority.authority_id is
'权限ID：authority_id';
insert into role_authority select 1,  generate_series (1, 11);

create table role_info (
                           role_id              SERIAL               not null,
                           role_name            VARCHAR(255)         null,
                           en_name              VARCHAR(255)         null,
                           update_time          TIMESTAMP            null,
                           constraint PK_ROLE_INFO primary key (role_id)
);
comment on table role_info is
'角色信息：role_info';

comment on column role_info.role_id is
'角色ID：role_id';

comment on column role_info.role_name is
'角色名称：role_name';

comment on column role_info.en_name is
'英文名称：en_name';

comment on column role_info.update_time is
'更新时间：update_time';

insert into role_info values
    (1, '超级管理员', 'superAdmin', now());

select nextval('role_info_role_id_seq');

/*==============================================================*/
/* Table: user_info                                             */
/*==============================================================*/
create table user_info (
   id              SERIAL               not null,
   user_code            VARCHAR(255)         null,
   display_name         VARCHAR(255)         null,
   password             VARCHAR(255)         null,
   authorization_code   VARCHAR(64)          null,
   head_img             VARCHAR(1024)        null,
   file_save_path       VARCHAR(1024)        null,
   has_auth             BOOL                 null,
   login_error_count    INT4                 null,
   locking              BOOL                 null,
   lock_time            TIMESTAMP            null,
   update_time          TIMESTAMP            null,
   has_manage           BOOLEAN              null,
   constraint PK_USER_INFO primary key (id)
);
comment on table user_info is
'用户信息：user_info';

comment on column user_info.id is
'用户ID：user_id';

comment on column user_info.user_code is
'登录账号：user_code';

comment on column user_info.display_name is
'显示名称：display_name';

comment on column user_info.password is
'登录密码：password';

comment on column user_info.authorization_code is
'授权码：authorization_code';

comment on column user_info.head_img is
'头像：head_img';

comment on column user_info.file_save_path is
'文件保存路径：file_save_path';

comment on column user_info.has_auth is
'是否有审核权限：has_auth';

comment on column user_info.login_error_count is
'登录错误次数：login_error_count';

comment on column user_info.locking is
'是否锁定：locking';

comment on column user_info.lock_time is
'锁定时间：lock_time';

comment on column user_info.update_time is
'更新时间：update_time';

comment on column user_info.has_manage is
'能否进入管理后台：has_manage';


/*==============================================================*/
/* Table: user_role                                             */
/*==============================================================*/
create table user_role (
                           role_id              INT4                 not null,
                           user_id              INT4                 not null,
                           constraint PK_USER_ROLE primary key (role_id, user_id)
);

comment on table user_role is
'用户角色：user_role';

comment on column user_role.role_id is
'角色ID：role_id';

comment on column user_role.user_id is
'用户ID：user_id';